import os
import json
import pkg_resources

from meggie.utilities.testing import BaseTestAction
from meggie.utilities.testing import create_test_experiment


class BaseStatisticsTestAction(BaseTestAction):

    def setup_experiment(self):
        self.experiment = create_test_experiment(
            self.dirpath, "test_experiment", n_subjects=10
        )
        self.experiment.activate_subject("sample_01-raw")

    def load_action_spec(self, action_name):
        action_path = pkg_resources.resource_filename("meggie_statistics", "actions")
        config_path = os.path.join(action_path, action_name, "configuration.json")
        with open(config_path, "r") as f:
            action_spec = json.load(f)
        return action_spec



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'permutationTestDialogUi.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtWidgets


class Ui_permutationTestDialog(object):
    def setupUi(self, permutationTestDialog):
        permutationTestDialog.setObjectName("permutationTestDialog")
        permutationTestDialog.resize(500, 716)
        self.gridLayout = QtWidgets.QGridLayout(permutationTestDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.scrollArea = QtWidgets.QScrollArea(permutationTestDialog)
        self.scrollArea.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollArea.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setSizeAdjustPolicy(
            QtWidgets.QAbstractScrollArea.AdjustToContents
        )
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 466, 885))
        self.scrollAreaWidgetContents.setMinimumSize(QtCore.QSize(0, 0))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.groupBoxStatistics = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxStatistics.setObjectName("groupBoxStatistics")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.groupBoxStatistics)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.labelClusterThreshold = QtWidgets.QLabel(self.groupBoxStatistics)
        self.labelClusterThreshold.setObjectName("labelClusterThreshold")
        self.gridLayout_6.addWidget(self.labelClusterThreshold, 0, 0, 1, 1)
        self.doubleSpinBoxClusterThreshold = QtWidgets.QDoubleSpinBox(
            self.groupBoxStatistics
        )
        self.doubleSpinBoxClusterThreshold.setDecimals(5)
        self.doubleSpinBoxClusterThreshold.setMaximum(1.0)
        self.doubleSpinBoxClusterThreshold.setSingleStep(0.001)
        self.doubleSpinBoxClusterThreshold.setProperty("value", 0.01)
        self.doubleSpinBoxClusterThreshold.setObjectName(
            "doubleSpinBoxClusterThreshold"
        )
        self.gridLayout_6.addWidget(self.doubleSpinBoxClusterThreshold, 0, 1, 1, 1)
        self.spinBoxNPermutations = QtWidgets.QSpinBox(self.groupBoxStatistics)
        self.spinBoxNPermutations.setMaximum(1000000000)
        self.spinBoxNPermutations.setSingleStep(100)
        self.spinBoxNPermutations.setProperty("value", 500)
        self.spinBoxNPermutations.setObjectName("spinBoxNPermutations")
        self.gridLayout_6.addWidget(self.spinBoxNPermutations, 2, 1, 1, 1)
        self.labelNPermutations = QtWidgets.QLabel(self.groupBoxStatistics)
        self.labelNPermutations.setObjectName("labelNPermutations")
        self.gridLayout_6.addWidget(self.labelNPermutations, 2, 0, 1, 1)
        self.labelClusterSignificance = QtWidgets.QLabel(self.groupBoxStatistics)
        self.labelClusterSignificance.setObjectName("labelClusterSignificance")
        self.gridLayout_6.addWidget(self.labelClusterSignificance, 1, 0, 1, 1)
        self.doubleSpinBoxClusterSignificance = QtWidgets.QDoubleSpinBox(
            self.groupBoxStatistics
        )
        self.doubleSpinBoxClusterSignificance.setDecimals(5)
        self.doubleSpinBoxClusterSignificance.setMaximum(1.0)
        self.doubleSpinBoxClusterSignificance.setSingleStep(0.001)
        self.doubleSpinBoxClusterSignificance.setProperty("value", 0.05)
        self.doubleSpinBoxClusterSignificance.setObjectName(
            "doubleSpinBoxClusterSignificance"
        )
        self.gridLayout_6.addWidget(self.doubleSpinBoxClusterSignificance, 1, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxStatistics, 3, 0, 1, 1)
        self.groupBoxChannel = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxChannel.setObjectName("groupBoxChannel")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.groupBoxChannel)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.comboBoxChannelName = QtWidgets.QComboBox(self.groupBoxChannel)
        self.comboBoxChannelName.setEnabled(False)
        self.comboBoxChannelName.setObjectName("comboBoxChannelName")
        self.gridLayout_3.addWidget(self.comboBoxChannelName, 2, 1, 1, 1)
        self.radioButtonChannelType = QtWidgets.QRadioButton(self.groupBoxChannel)
        self.radioButtonChannelType.setChecked(True)
        self.radioButtonChannelType.setObjectName("radioButtonChannelType")
        self.gridLayout_3.addWidget(self.radioButtonChannelType, 0, 0, 1, 1)
        self.comboBoxChannelType = QtWidgets.QComboBox(self.groupBoxChannel)
        self.comboBoxChannelType.setObjectName("comboBoxChannelType")
        self.gridLayout_3.addWidget(self.comboBoxChannelType, 0, 1, 1, 1)
        self.radioButtonChannelName = QtWidgets.QRadioButton(self.groupBoxChannel)
        self.radioButtonChannelName.setObjectName("radioButtonChannelName")
        self.gridLayout_3.addWidget(self.radioButtonChannelName, 2, 0, 1, 1)
        self.radioButtonChannelGroup = QtWidgets.QRadioButton(self.groupBoxChannel)
        self.radioButtonChannelGroup.setObjectName("radioButtonChannelGroup")
        self.gridLayout_3.addWidget(self.radioButtonChannelGroup, 1, 0, 1, 1)
        self.comboBoxChannelGroup = QtWidgets.QComboBox(self.groupBoxChannel)
        self.comboBoxChannelGroup.setEnabled(False)
        self.comboBoxChannelGroup.setObjectName("comboBoxChannelGroup")
        self.gridLayout_3.addWidget(self.comboBoxChannelGroup, 1, 1, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxChannel, 0, 0, 1, 1)
        self.groupBoxFrequency = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxFrequency.setObjectName("groupBoxFrequency")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxFrequency)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.doubleSpinBoxFrequencyFmin = QtWidgets.QDoubleSpinBox(
            self.groupBoxFrequency
        )
        self.doubleSpinBoxFrequencyFmin.setEnabled(False)
        self.doubleSpinBoxFrequencyFmin.setDecimals(3)
        self.doubleSpinBoxFrequencyFmin.setMinimum(-10000.0)
        self.doubleSpinBoxFrequencyFmin.setMaximum(10000.0)
        self.doubleSpinBoxFrequencyFmin.setSingleStep(0.1)
        self.doubleSpinBoxFrequencyFmin.setObjectName("doubleSpinBoxFrequencyFmin")
        self.gridLayout_5.addWidget(self.doubleSpinBoxFrequencyFmin, 0, 2, 1, 1)
        self.radioButtonFrequencyEnabled = QtWidgets.QRadioButton(
            self.groupBoxFrequency
        )
        self.radioButtonFrequencyEnabled.setObjectName("radioButtonFrequencyEnabled")
        self.gridLayout_5.addWidget(self.radioButtonFrequencyEnabled, 0, 0, 1, 1)
        self.labelFrequencyFmin = QtWidgets.QLabel(self.groupBoxFrequency)
        self.labelFrequencyFmin.setObjectName("labelFrequencyFmin")
        self.gridLayout_5.addWidget(self.labelFrequencyFmin, 0, 1, 1, 1)
        self.labelFrequencyFmax = QtWidgets.QLabel(self.groupBoxFrequency)
        self.labelFrequencyFmax.setObjectName("labelFrequencyFmax")
        self.gridLayout_5.addWidget(self.labelFrequencyFmax, 1, 1, 1, 1)
        self.radioButtonFrequencyDisabled = QtWidgets.QRadioButton(
            self.groupBoxFrequency
        )
        self.radioButtonFrequencyDisabled.setChecked(True)
        self.radioButtonFrequencyDisabled.setObjectName("radioButtonFrequencyDisabled")
        self.gridLayout_5.addWidget(self.radioButtonFrequencyDisabled, 2, 0, 1, 1)
        self.doubleSpinBoxFrequencyFmax = QtWidgets.QDoubleSpinBox(
            self.groupBoxFrequency
        )
        self.doubleSpinBoxFrequencyFmax.setEnabled(False)
        self.doubleSpinBoxFrequencyFmax.setDecimals(3)
        self.doubleSpinBoxFrequencyFmax.setMinimum(-10000.0)
        self.doubleSpinBoxFrequencyFmax.setMaximum(10000.0)
        self.doubleSpinBoxFrequencyFmax.setSingleStep(0.1)
        self.doubleSpinBoxFrequencyFmax.setObjectName("doubleSpinBoxFrequencyFmax")
        self.gridLayout_5.addWidget(self.doubleSpinBoxFrequencyFmax, 1, 2, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxFrequency, 2, 0, 1, 1)
        self.groupBoxGroups = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxGroups.setObjectName("groupBoxGroups")
        self.formLayout = QtWidgets.QFormLayout(self.groupBoxGroups)
        self.formLayout.setObjectName("formLayout")
        self.pushButtonGroups = QtWidgets.QPushButton(self.groupBoxGroups)
        self.pushButtonGroups.setObjectName("pushButtonGroups")
        self.formLayout.setWidget(
            0, QtWidgets.QFormLayout.SpanningRole, self.pushButtonGroups
        )
        self.listWidgetGroups = QtWidgets.QListWidget(self.groupBoxGroups)
        self.listWidgetGroups.setObjectName("listWidgetGroups")
        self.formLayout.setWidget(
            1, QtWidgets.QFormLayout.SpanningRole, self.listWidgetGroups
        )
        self.gridLayout_2.addWidget(self.groupBoxGroups, 4, 0, 1, 1)
        self.groupBoxTime = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxTime.setObjectName("groupBoxTime")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groupBoxTime)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.doubleSpinBoxTimeTmin = QtWidgets.QDoubleSpinBox(self.groupBoxTime)
        self.doubleSpinBoxTimeTmin.setEnabled(False)
        self.doubleSpinBoxTimeTmin.setDecimals(3)
        self.doubleSpinBoxTimeTmin.setMinimum(-10000.0)
        self.doubleSpinBoxTimeTmin.setMaximum(10000.0)
        self.doubleSpinBoxTimeTmin.setSingleStep(0.1)
        self.doubleSpinBoxTimeTmin.setObjectName("doubleSpinBoxTimeTmin")
        self.gridLayout_4.addWidget(self.doubleSpinBoxTimeTmin, 0, 2, 1, 1)
        self.radioButtonTimeDisabled = QtWidgets.QRadioButton(self.groupBoxTime)
        self.radioButtonTimeDisabled.setChecked(True)
        self.radioButtonTimeDisabled.setObjectName("radioButtonTimeDisabled")
        self.gridLayout_4.addWidget(self.radioButtonTimeDisabled, 2, 0, 1, 1)
        self.radioButtonTimeEnabled = QtWidgets.QRadioButton(self.groupBoxTime)
        self.radioButtonTimeEnabled.setObjectName("radioButtonTimeEnabled")
        self.gridLayout_4.addWidget(self.radioButtonTimeEnabled, 0, 0, 1, 1)
        self.labelTimeTmin = QtWidgets.QLabel(self.groupBoxTime)
        self.labelTimeTmin.setObjectName("labelTimeTmin")
        self.gridLayout_4.addWidget(self.labelTimeTmin, 0, 1, 1, 1)
        self.labelTimeTmax = QtWidgets.QLabel(self.groupBoxTime)
        self.labelTimeTmax.setObjectName("labelTimeTmax")
        self.gridLayout_4.addWidget(self.labelTimeTmax, 1, 1, 1, 1)
        self.doubleSpinBoxTimeTmax = QtWidgets.QDoubleSpinBox(self.groupBoxTime)
        self.doubleSpinBoxTimeTmax.setEnabled(False)
        self.doubleSpinBoxTimeTmax.setDecimals(3)
        self.doubleSpinBoxTimeTmax.setMinimum(-10000.0)
        self.doubleSpinBoxTimeTmax.setMaximum(10000.0)
        self.doubleSpinBoxTimeTmax.setSingleStep(0.1)
        self.doubleSpinBoxTimeTmax.setObjectName("doubleSpinBoxTimeTmax")
        self.gridLayout_4.addWidget(self.doubleSpinBoxTimeTmax, 1, 2, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxTime, 1, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(
            20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding
        )
        self.gridLayout_2.addItem(spacerItem, 6, 0, 1, 1)
        self.groupBoxDesign = QtWidgets.QGroupBox(self.scrollAreaWidgetContents)
        self.groupBoxDesign.setObjectName("groupBoxDesign")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.groupBoxDesign)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.radioButtonWithinSubjects = QtWidgets.QRadioButton(self.groupBoxDesign)
        self.radioButtonWithinSubjects.setChecked(True)
        self.radioButtonWithinSubjects.setObjectName("radioButtonWithinSubjects")
        self.gridLayout_7.addWidget(self.radioButtonWithinSubjects, 0, 0, 1, 1)
        self.radioButtonBetweenSubjects = QtWidgets.QRadioButton(self.groupBoxDesign)
        self.radioButtonBetweenSubjects.setObjectName("radioButtonBetweenSubjects")
        self.gridLayout_7.addWidget(self.radioButtonBetweenSubjects, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.groupBoxDesign, 5, 0, 1, 1)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.gridLayout.addWidget(self.scrollArea, 0, 0, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem1 = QtWidgets.QSpacerItem(
            40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum
        )
        self.horizontalLayout.addItem(spacerItem1)
        self.pushButtonCancel = QtWidgets.QPushButton(permutationTestDialog)
        self.pushButtonCancel.setObjectName("pushButtonCancel")
        self.horizontalLayout.addWidget(self.pushButtonCancel)
        self.pushButtonApply = QtWidgets.QPushButton(permutationTestDialog)
        self.pushButtonApply.setObjectName("pushButtonApply")
        self.horizontalLayout.addWidget(self.pushButtonApply)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)

        self.retranslateUi(permutationTestDialog)
        self.pushButtonCancel.clicked.connect(permutationTestDialog.reject)  # type: ignore
        self.pushButtonApply.clicked.connect(permutationTestDialog.accept)  # type: ignore
        self.radioButtonFrequencyDisabled.toggled["bool"].connect(self.doubleSpinBoxFrequencyFmax.setDisabled)  # type: ignore
        self.radioButtonFrequencyDisabled.toggled["bool"].connect(self.doubleSpinBoxFrequencyFmin.setDisabled)  # type: ignore
        self.radioButtonTimeDisabled.toggled["bool"].connect(self.doubleSpinBoxTimeTmin.setDisabled)  # type: ignore
        self.radioButtonTimeDisabled.toggled["bool"].connect(self.doubleSpinBoxTimeTmax.setDisabled)  # type: ignore
        self.radioButtonChannelType.toggled["bool"].connect(self.comboBoxChannelGroup.setDisabled)  # type: ignore
        self.radioButtonChannelType.toggled["bool"].connect(self.comboBoxChannelName.setDisabled)  # type: ignore
        self.radioButtonChannelGroup.toggled["bool"].connect(self.comboBoxChannelType.setDisabled)  # type: ignore
        self.radioButtonChannelGroup.toggled["bool"].connect(self.comboBoxChannelName.setDisabled)  # type: ignore
        self.radioButtonChannelName.toggled["bool"].connect(self.comboBoxChannelGroup.setDisabled)  # type: ignore
        self.radioButtonChannelName.toggled["bool"].connect(self.comboBoxChannelType.setDisabled)  # type: ignore
        QtCore.QMetaObject.connectSlotsByName(permutationTestDialog)

    def retranslateUi(self, permutationTestDialog):
        _translate = QtCore.QCoreApplication.translate
        permutationTestDialog.setWindowTitle(
            _translate("permutationTestDialog", "Meggie - Permutation tests")
        )
        self.groupBoxStatistics.setTitle(
            _translate("permutationTestDialog", "Statistics:")
        )
        self.labelClusterThreshold.setText(
            _translate("permutationTestDialog", "Cluster threshold (p):")
        )
        self.labelNPermutations.setText(
            _translate("permutationTestDialog", "Number of permutations:")
        )
        self.labelClusterSignificance.setText(
            _translate("permutationTestDialog", "Cluster significance (p):")
        )
        self.groupBoxChannel.setTitle(
            _translate("permutationTestDialog", "Limit channels:")
        )
        self.radioButtonChannelType.setText(
            _translate("permutationTestDialog", "Channel type:")
        )
        self.radioButtonChannelName.setText(
            _translate("permutationTestDialog", "Channel:")
        )
        self.radioButtonChannelGroup.setText(
            _translate("permutationTestDialog", "Channel group:")
        )
        self.groupBoxFrequency.setTitle(
            _translate("permutationTestDialog", "Limit in frequency:")
        )
        self.radioButtonFrequencyEnabled.setText(
            _translate("permutationTestDialog", "Limit to:")
        )
        self.labelFrequencyFmin.setText(_translate("permutationTestDialog", "Fmin:"))
        self.labelFrequencyFmax.setText(_translate("permutationTestDialog", "Fmax:"))
        self.radioButtonFrequencyDisabled.setText(
            _translate("permutationTestDialog", "Do not limit")
        )
        self.groupBoxGroups.setTitle(_translate("permutationTestDialog", "Groups:"))
        self.pushButtonGroups.setText(
            _translate("permutationTestDialog", "Select groups..")
        )
        self.groupBoxTime.setTitle(
            _translate("permutationTestDialog", "Limit in time:")
        )
        self.radioButtonTimeDisabled.setText(
            _translate("permutationTestDialog", "Do not limit")
        )
        self.radioButtonTimeEnabled.setText(
            _translate("permutationTestDialog", "Limit to:")
        )
        self.labelTimeTmin.setText(_translate("permutationTestDialog", "Tmin:"))
        self.labelTimeTmax.setText(_translate("permutationTestDialog", "Tmax:"))
        self.groupBoxDesign.setTitle(_translate("permutationTestDialog", "Design:"))
        self.radioButtonWithinSubjects.setText(
            _translate("permutationTestDialog", "Within-subjects")
        )
        self.radioButtonBetweenSubjects.setText(
            _translate("permutationTestDialog", "Between-subjects")
        )
        self.pushButtonCancel.setText(_translate("permutationTestDialog", "Cancel"))
        self.pushButtonApply.setText(_translate("permutationTestDialog", "Apply"))



from meggie_statistics.utilities.testing import BaseStatisticsTestAction
from meggie_statistics.actions.spectrum_permutation_test import PermutationTest
from meggie_statistics.utilities.dialogs.permutationTestDialogMain import (
    PermutationTestDialog,
)


class TestSpectrumPermutationTest(BaseStatisticsTestAction):
    def test_spectrum_permutation_test(self):

        data = {"outputs": {"spectrum": ["Spectrum"]}}

        self.run_action(
            action_name="spectrum_permutation_test",
            handler=PermutationTest,
            data=data,
            patch_paths=["meggie_statistics.actions.spectrum_permutation_test"],
        )
        dialog = self.find_dialog(PermutationTestDialog)

        dialog.ui.radioButtonWithinSubjects.setChecked(False)
        dialog.groups = {
            "1": [
                "sample_01-raw",
                "sample_02-raw",
                "sample_03-raw",
                "sample_04-raw",
                "sample_05-raw",
            ],
            "2": [
                "sample_06-raw",
                "sample_07-raw",
                "sample_08-raw",
                "sample_09-raw",
                "sample_10-raw",
            ],
        }
        dialog.accept()



""" Contains a class for logic of the permutation test dialog.
"""

from PyQt5 import QtWidgets


from meggie.utilities.channels import get_channels_by_type

from meggie.utilities.messaging import messagebox

from meggie_statistics.utilities.dialogs.permutationTestDialogUi import (
    Ui_permutationTestDialog,
)
from meggie.utilities.dialogs.groupSelectionDialogMain import GroupSelectionDialog


class PermutationTestDialog(QtWidgets.QDialog):
    """Contains logic for the permutation test dialog."""

    def __init__(
        self,
        experiment,
        parent,
        handler,
        meggie_item,
        limit_frequency=False,
        limit_time=False,
        limit_channel=True,
    ):
        QtWidgets.QDialog.__init__(self, parent)
        self.ui = Ui_permutationTestDialog()
        self.ui.setupUi(self)

        self.handler = handler
        self.experiment = experiment

        ch_types = get_channels_by_type(meggie_item.info).keys()
        self.channel_group_options = []

        # add eeg if relevant
        if experiment.channel_groups.get("eeg") and "eeg" in ch_types:
            for ch_group_name, _ in experiment.channel_groups["eeg"].items():
                self.channel_group_options.append(("eeg", ch_group_name))

        # add grad if relevant
        if experiment.channel_groups.get("meg") and "grad" in ch_types:
            for ch_group_name, _ in experiment.channel_groups["meg"].items():
                self.channel_group_options.append(("grad", ch_group_name))

        # add mag if relevant
        if experiment.channel_groups.get("meg") and "mag" in ch_types:
            for ch_group_name, _ in experiment.channel_groups["meg"].items():
                self.channel_group_options.append(("mag", ch_group_name))

        self.limit_frequency = limit_frequency
        self.limit_time = limit_time
        self.limit_channel = limit_channel

        if not limit_time:
            self.ui.groupBoxTime.hide()
        else:
            self.ui.doubleSpinBoxTimeTmin.setValue(meggie_item.times[0])
            self.ui.doubleSpinBoxTimeTmax.setValue(meggie_item.times[-1])

        if not limit_frequency:
            self.ui.groupBoxFrequency.hide()
        else:
            self.ui.doubleSpinBoxFrequencyFmin.setValue(meggie_item.freqs[0])
            self.ui.doubleSpinBoxFrequencyFmax.setValue(meggie_item.freqs[-1])

        if not limit_channel:
            self.ui.groupBoxChannel.hide()
        else:
            for ch_type in ch_types:
                self.ui.comboBoxChannelType.addItem(ch_type)
            for ch_name in meggie_item.info["ch_names"]:
                self.ui.comboBoxChannelName.addItem(ch_name)
            for ch_type, ch_group_name in self.channel_group_options:
                self.ui.comboBoxChannelGroup.addItem(f"{ch_type} - {ch_group_name}")

        self.meggie_item = meggie_item

        self.groups = {}

    def on_pushButtonGroups_clicked(self, checked=None):
        if checked is None:
            return

        def handler(groups):
            if not groups:
                return

            self.groups = groups
            self.ui.listWidgetGroups.clear()
            for key, names in sorted(groups.items(), key=lambda x: x[0]):
                for name in sorted(names):
                    item_name = str(key) + ": " + str(name)
                    self.ui.listWidgetGroups.addItem(item_name)

        dialog = GroupSelectionDialog(self.experiment, self, handler)
        dialog.show()

    def accept(self):
        if not self.groups:
            messagebox(self, "You should select some groups first")
            return

        if self.ui.radioButtonWithinSubjects.isChecked():
            design = "within-subjects"
        else:
            design = "between-subjects"

        if design == "between-subjects" and len(self.groups) <= 1:
            messagebox(
                self, "At least two groups are needed for between-subjects design"
            )
            return

        if design == "within-subjects":
            conditions = self.meggie_item.content.keys()
            if len(conditions) <= 1:
                messagebox(
                    self,
                    "At least two conditions are needed for within-subjects design",
                )
                return

        time_limits = None
        frequency_limits = None
        location_limits = None

        if self.limit_time and self.ui.radioButtonTimeEnabled.isChecked():
            tmin = self.ui.doubleSpinBoxTimeTmin.value()
            tmax = self.ui.doubleSpinBoxTimeTmax.value()
            time_limits = tmin, tmax

        if self.limit_frequency and self.ui.radioButtonFrequencyEnabled.isChecked():
            fmin = self.ui.doubleSpinBoxFrequencyFmin.value()
            fmax = self.ui.doubleSpinBoxFrequencyFmax.value()
            frequency_limits = fmin, fmax

        if self.limit_channel and self.ui.radioButtonChannelType.isChecked():
            location_limits = ("ch_type", self.ui.comboBoxChannelType.currentText())
        if self.limit_channel and self.ui.radioButtonChannelName.isChecked():
            location_limits = ("ch_name", self.ui.comboBoxChannelName.currentText())
        if self.limit_channel and self.ui.radioButtonChannelGroup.isChecked():
            idx = self.ui.comboBoxChannelGroup.currentIndex()
            ch_group_choice = self.channel_group_options[idx]
            location_limits = ("ch_group", ch_group_choice)

        threshold = self.ui.doubleSpinBoxClusterThreshold.value()
        significance = self.ui.doubleSpinBoxClusterSignificance.value()
        n_permutations = self.ui.spinBoxNPermutations.value()

        params = {}
        params["groups"] = self.groups
        params["time_limits"] = time_limits
        params["frequency_limits"] = frequency_limits
        params["location_limits"] = location_limits
        params["threshold"] = threshold
        params["significance"] = significance
        params["n_permutations"] = n_permutations
        params["design"] = design

        self.handler(params)
        self.close()



""" Contains implementation for evoked permutation tests
"""

from meggie.utilities.messaging import exc_messagebox

from meggie.mainwindow.dynamic import Action
from meggie.mainwindow.dynamic import subject_action

from meggie_statistics.actions.evoked_permutation_test.controller.evoked import (
    run_permutation_test,
)
from meggie_statistics.utilities.dialogs.permutationTestDialogMain import (
    PermutationTestDialog,
)


class PermutationTest(Action):
    """Runs permutation tests for evoked items."""

    def run(self):
        try:
            selected_name = self.data["outputs"]["evoked"][0]
        except IndexError:
            return

        meggie_item = self.experiment.active_subject.evoked[selected_name]

        def handle_close(params):
            params["name"] = selected_name
            try:
                self.handler(self.experiment.active_subject, params)
            except Exception as exc:
                exc_messagebox(self.window, exc)

        dialog = PermutationTestDialog(
            self.experiment, self.window, handle_close, meggie_item, limit_time=True
        )
        dialog.show()

    @subject_action
    def handler(self, subject, params):
        """ """
        run_permutation_test(
            self.experiment,
            self.window,
            params["name"],
            params["groups"],
            params["time_limits"],
            params["frequency_limits"],
            params["location_limits"],
            params["threshold"],
            params["significance"],
            params["n_permutations"],
            params["design"],
        )



""" Contains generic functions for permutation test computations.
"""

import logging
from collections import OrderedDict

import mne

import numpy as np
import matplotlib.pyplot as plt
import scipy.stats
import scipy

from meggie.utilities.threading import threaded
from meggie.utilities.channels import average_to_channel_groups
from meggie.utilities.channels import get_channels_by_type
from meggie.utilities.channels import pairless_grads
from meggie.utilities.messaging import questionbox


def _prepare_channel_info(
    item_type, item_name, meggie_item, location_limits, groups, experiment
):
    if location_limits[0] == "ch_type":
        # find channels shared by all subjects
        ch_names = []
        for group_key, group_subjects in groups.items():
            for subject_name in group_subjects:
                subject = experiment.subjects.get(subject_name)
                sub_item = getattr(subject, item_type).get(item_name)
                if sub_item:
                    ch_names.append(tuple(sub_item.ch_names))
        common_ch_names = list(set.intersection(*map(set, ch_names)))

        # filter to selected channel type
        ch_type = location_limits[1]
        chs_by_type = get_channels_by_type(meggie_item.info)
        common_ch_names = [
            ch_name for ch_name in common_ch_names if ch_name in chs_by_type[ch_type]
        ]

        # remove pairless grads
        if ch_type == "grad":
            pairless_idxs = pairless_grads(common_ch_names)
            common_ch_names = [
                ch_name
                for ch_idx, ch_name in enumerate(common_ch_names)
                if ch_idx not in pairless_idxs
            ]

        # update info to match valid channels
        info_ch_names = meggie_item.info["ch_names"]
        picks = [
            ch_idx
            for ch_idx, ch_name in enumerate(info_ch_names)
            if ch_name in common_ch_names
        ]
        info = mne.pick_info(meggie_item.info.copy(), sel=picks)

        # find adjacency matching adjacency
        adjacency, adjacency_ch_names = mne.channels.find_ch_adjacency(info, ch_type)
        kept_idxs = [
            idx
            for idx in range(adjacency.shape[0])
            if adjacency_ch_names[idx] in common_ch_names
        ]

        adjacency_arr = adjacency.toarray()[kept_idxs][:, kept_idxs]
        adjacency = scipy.sparse.csr_matrix(adjacency_arr)
    elif location_limits[0] == "ch_group":
        adjacency = scipy.sparse.csr_matrix([1])
        info = meggie_item.info
    else:
        adjacency = scipy.sparse.csr_matrix([1])
        info = meggie_item.info

    return common_ch_names, info, adjacency


def _average_over_frequency(data, freqs, frequency_limits, data_format):
    fmin_idx = np.where(freqs >= frequency_limits[0])[0][0]
    fmax_idx = np.where(freqs <= frequency_limits[1])[0][-1]
    swapped = np.swapaxes(data, data_format.index("freqs"), 0)
    averaged = np.mean(swapped[fmin_idx:fmax_idx], axis=0)[np.newaxis, :]
    return np.swapaxes(averaged, data_format.index("freqs"), 0)


def _select_data_by_location(
    data, info, location_limits, data_format, subject_item, common_ch_names, experiment
):
    if location_limits[0] == "ch_name":
        ch_names = subject_item.ch_names
        ch_idx = ch_names.index(location_limits[1])
        swapped = np.swapaxes(data, data_format.index("locations"), 0)
        selected = data[ch_idx][np.newaxis, :]
        return np.swapaxes(selected, data_format.index("locations"), 0)

    elif location_limits[0] == "ch_group":
        ch_names = subject_item.ch_names
        swapped = np.swapaxes(data, data_format.index("locations"), 0)
        labels, averaged = average_to_channel_groups(
            data, info, ch_names, experiment.channel_groups
        )
        averaged_idx = [
            idx
            for idx, lbl in enumerate(labels)
            if lbl[0] == location_limits[1][0] and lbl[1] == location_limits[1][1]
        ][0]
        selected = averaged[averaged_idx][np.newaxis, :]
        return np.swapaxes(selected, data_format.index("locations"), 0)

    elif location_limits[0] == "ch_type":
        # filter to channel type
        ch_idxs = np.array(
            [
                ch_idx
                for ch_idx, ch_name in enumerate(subject_item.info["ch_names"])
                if ch_name in common_ch_names
            ]
        )
        swapped = np.swapaxes(data, data_format.index("locations"), 0)
        selected = swapped[ch_idxs]
        return np.swapaxes(selected, data_format.index("locations"), 0)


def _average_over_time(data, times, time_limits, data_format):
    tmin_idx = np.where(times >= time_limits[0])[0][0]
    tmax_idx = np.where(times <= time_limits[1])[0][-1]
    swapped = np.swapaxes(data, data_format.index("times"), 0)
    averaged = np.mean(swapped[tmin_idx:tmax_idx], axis=0)[np.newaxis, :]
    return np.swapaxes(averaged, data_format.index("times"), 0)


@threaded
def prepare_data_for_permutation(
    experiment,
    design,
    groups,
    item_type,
    item_name,
    location_limits,
    time_limits,
    frequency_limits,
    data_format=("locations", "freqs", "times"),
):
    """Prepares data for a permutation test that is run with subjects as samples.

    Basically,
    1) reorders axes,
    2) averages and selects data according to limits,
    3) and updates info to contain correct set of channels.

    Returns dict with conditions as keys if between-subjects design is used. Each condition
    is thus handled separately. If within-subjects design is used, returns dict with groups as keys.
    Is supposed to be as general as possible to allow usage for different kinds of datatypes.

    Parameters
    ----------
    experiment : meggie.experiment.Experiment
        The experiment to collect the data from.
    design : str
        ANOVA design, either 'within-subjects' or 'between-subjects'.
    groups : dict
        Contains subjects organized to groups.
    item_type : str
        The datatype that is collected from subjects, e.g. "evoked".
    item_name : str
        Name of the data dataobjects to be collected, e.g. "Auditory'.
    location_limits : tuple
        Can be ('ch_name': ch_name) or ('ch_type': ch_type) for some ch_name
        or ch_type.
    time_limits : tuple
        Should be (tmin, tmax) in seconds, e.g (0, 0.3).
    frequency_limits : tuple
        Should be (fmin, fmax) in Hz, e.g. (8, 13).
    data_formats : tuple
        Specifies how the data is organizes, i.e the order of axes.

    Returns
    -------
    mne.Info
        The modified info structure.
    dict
        The prepared data organized by condition or group depending on the design.
    scipy.sparse.csr_matrix
        The adjacency matrix, properly modified to match the info and the data.
    """
    logging.getLogger("ui_logger").info("Preparing data for permutations..")
    meggie_item = getattr(experiment.active_subject, item_type)[item_name]
    conditions = list(meggie_item.content.keys())
    groups = OrderedDict(sorted(groups.items()))

    for group_key, group in groups.items():
        if len(group) <= 1:
            raise Exception(f"Group {group_key} must have more than 1 subjects.")

    common_ch_names, info, adjacency = _prepare_channel_info(
        item_type, item_name, meggie_item, location_limits, groups, experiment
    )

    final_data = {}
    if design == "between-subjects":

        for condition in conditions:
            cond_data = []
            for group in groups.values():
                group_data = []
                for subject_name in group:
                    subject = experiment.subjects[subject_name]
                    subject_item = getattr(subject, item_type).get(item_name)
                    if not subject_item:
                        message = (
                            "Skipping " + subject_name + " (no " + str(item_type) + ")"
                        )
                        logging.getLogger("ui_logger").warning(message)

                    data = subject_item.data[condition]

                    if frequency_limits is not None:
                        data = _average_over_frequency(
                            data, subject_item.freqs, frequency_limits, data_format
                        )

                    data = _select_data_by_location(
                        data,
                        info,
                        location_limits,
                        data_format,
                        subject_item,
                        common_ch_names,
                        experiment,
                    )

                    if time_limits is not None:
                        data = _average_over_time(
                            data, subject_item.times, time_limits, data_format
                        )

                    # move locations dim to last place
                    data = np.rollaxis(data, data_format.index("locations"), data.ndim)

                    group_data.append(data)
                cond_data.append(np.array(group_data))
            final_data[condition] = cond_data

    elif design == "within-subjects":
        for group_key, group in groups.items():
            group_data = []
            for subject_idx, subject_name in enumerate(group):
                subject = experiment.subjects[subject_name]
                subject_item = getattr(subject, item_type).get(item_name)
                if not subject_item:
                    message = (
                        "Skipping " + subject_name + " (no " + str(item_type) + ")"
                    )
                    logging.getLogger("ui_logger").warning(message)

                cond_data = []
                for condition_idx, condition in enumerate(conditions):

                    data = subject_item.data[condition]

                    if frequency_limits is not None:
                        data = _average_over_frequency(
                            data, subject_item.freqs, frequency_limits, data_format
                        )

                    data = _select_data_by_location(
                        data,
                        info,
                        location_limits,
                        data_format,
                        subject_item,
                        common_ch_names,
                        experiment,
                    )

                    if time_limits is not None:
                        data = _average_over_time(
                            data, subject_item.times, time_limits, data_format
                        )

                    cond_data.append(data)
                group_data.append(cond_data)

            # The format needed is:
            # (n_conds, n_subjects, ..., n_channels)
            swapped = np.swapaxes(np.array(group_data), 0, 1)
            rolled = np.rollaxis(
                swapped, 2 + data_format.index("locations"), swapped.ndim
            )
            final_data[group_key] = rolled

    return info, final_data, adjacency


@threaded
def permutation_analysis(
    data,
    design,
    conditions,
    groups,
    threshold,
    adjacency,
    n_permutations,
    random_state=None,
):
    """Run permutation tests with properly prepared data.

    Uses either between-subjects or within-subjects ANOVA
    for mass univariate tests.

    Parameters
    ----------
    data : dict
        Data formatted as in prepare_data_for_permutation.
    design : str
        The ANOVA design, can be either within-subjects or between-subjects.
    conditions : list
        The conditions present in the data.
    groups : list
        The groups present in the data.
    threshold : float
        A probability threshold (e.g 0.01) used to find the critical value
        for univariate F-tests.
    adjacency : scipy.sparse.csr_matrix
        Adjacency matrix for correcting for multiple comparisons.
    n_permutations : int
        The number of permutations used.
    random_state : None, int or np.RandomState
        A seed to be passed to the permutation test procedure.

    Returns
    -------
    dict
        The permutation test results as in mne.stats.permutation_cluster_test
        organized as dict with conditions or groups as keys depending on the
        design.
    """

    # if 3d (or higher) clusters, create combined for adjacency
    # as mne flattens all but the first dimension
    sample_shape = list(data.values())[0][0][0].shape
    if len(sample_shape) > 2:
        combined_adjacency = mne.stats.combine_adjacency(*sample_shape[1:-1], adjacency)
    else:
        combined_adjacency = adjacency

    logging.getLogger("ui_logger").info("Running permutation tests..")
    results = {}
    if design == "between-subjects":
        for condition in conditions:
            X = data[condition]

            res = mne.stats.permutation_cluster_test(
                X=X,
                threshold=threshold,
                n_permutations=n_permutations,
                adjacency=combined_adjacency,
                verbose="warning",
                out_type="indices",
                seed=random_state,
            )
            results[condition] = res
    else:
        for group_key, group in groups.items():
            X = data[group_key]
            factor_levels, effects = [len(conditions)], "A"
            f_thresh = mne.stats.f_threshold_mway_rm(
                len(group), factor_levels, effects, threshold
            )

            # data before: (n_conditions, n_subjects, ..., n_locations)
            # give to permutation_cluster test as: (n_conditions * n_subjects, ..., n_locations)
            # and format again properly at stat_fun for f_mway_rm: (n_subjects, n_conditions, n_others)
            # np.allclose(np.array(np.split(np.concatenate(X, axis=0), X.shape[0])), X) == True

            n_conditions = X.shape[0]

            def stat_fun(*args):
                sample = args[0]
                formatted = np.swapaxes(np.array(np.split(sample, n_conditions)), 0, 1)
                return mne.stats.f_mway_rm(
                    formatted,
                    factor_levels=factor_levels,
                    effects=effects,
                    return_pvals=False,
                )[0]

            res = mne.stats.permutation_cluster_test(
                X=[np.concatenate(X, axis=0)],
                adjacency=combined_adjacency,
                threshold=f_thresh,
                stat_fun=stat_fun,
                n_permutations=n_permutations,
                verbose="warning",
                out_type="indices",
                seed=random_state,
            )
            results[group_key] = res
    return results


def report_permutation_results(
    results,
    design,
    selected_name,
    significance,
    location_limits=None,
    frequency_limits=None,
    time_limits=None,
):
    """Logs a text report on the permutation results.

    Parameters
    ----------
    results : dict
        Results as in the permutation_analysis.
    design : str
        The ANOVA design, can be 'within-subjects' or 'between-subjects'.
    selected_name : str
        Name of the data object.
    significance : float
        Print info only on the clusters that have p values smaller than this.
    location_limits : tuple
        Can be ('ch_name': ch_name) or ('ch_type': ch_type) for some ch_name
        or ch_type.
    time_limits : tuple
        Should be (tmin, tmax) in seconds, e.g (0, 0.3).
    frequency_limits : tuple
        Should be (fmin, fmax) in Hz, e.g. (8, 13).

    """
    logger = logging.getLogger("ui_logger")
    logger.info("Permutation tests for " + str(selected_name) + " completed.")
    if location_limits:
        logger.info("Location limits: " + str(location_limits))
    if frequency_limits:
        logger.info("Frequency limits: " + str(frequency_limits))
    if time_limits:
        logger.info("Time limits: " + str(time_limits))

    for key, res in results.items():
        n_clusters = len(res[2])
        sign_mask = np.where(res[2] < significance)[0]
        n_sign_clusters = len(sign_mask)

        if design == "within-subjects":
            message = "Found {0} clusters ({1} significant) for group {2}."
        else:
            message = "Found {0} clusters ({1} significant) for condition {2}."
        logger.info(message.format(n_clusters, n_sign_clusters, key))

        for cluster_idx in range(n_sign_clusters):
            pvalue = res[2][sign_mask][cluster_idx]
            message = "Cluster {0} has p value: {1}."
            logger.info(message.format(cluster_idx + 1, pvalue))


def plot_permutation_results(
    results,
    significance,
    window,
    location_limits=None,
    time_limits=None,
    frequency_limits=None,
    location_fun=None,
    time_fun=None,
    frequency_fun=None,
):
    """Plots permutation results on figures.

    results : dict
        Results as in the permutation_analysis.
    significance : float
        Print info only on the clusters that have p values smaller than this.
    window : QMainWindow or QDialog
        Window to use as a parent for a questionbox.
    location_limits : tuple
        Can be ('ch_name': ch_name) or ('ch_type': ch_type) for some ch_name
        or ch_type.
    time_limits : tuple
        Should be (tmin, tmax) in seconds, e.g (0, 0.3).
    frequency_limits : tuple
        Should be (fmin, fmax) in Hz, e.g. (8, 13).
    location_fun : function
        Function that is called to do actually plot the cluster in the location space.
    time_fun : function
        Function that is called to actually plot the cluster in the time space.
    frequency_fun  function
        Function that is called to actually plot the cluster in the frequency space.
    """
    for key, res in results.items():
        sign_mask = np.where(res[2] < significance)[0]
        n_sign_clusters = len(sign_mask)

        def plot():
            for sign_idx in range(n_sign_clusters):
                cluster = res[1][sign_mask[sign_idx]]
                pvalue = res[2][sign_mask[sign_idx]]
                if frequency_limits is None:
                    if frequency_fun:
                        frequency_fun(sign_idx, cluster, pvalue, key)
                if time_limits is None:
                    if time_fun:
                        time_fun(sign_idx, cluster, pvalue, key)
                if location_limits is None or location_limits[0] == "ch_type":
                    if location_fun:
                        location_fun(sign_idx, cluster, pvalue, key)
                plt.show()

        if n_sign_clusters > 10:

            def handler(accepted):
                if accepted:
                    plot()

            message = (
                str(n_sign_clusters)
                + " significant clusters were found for "
                + str(key)
                + ". Do you want to plot all of them?"
            )
            questionbox(window, message, handler)
        else:
            plot()



""" Contains controlling logic for the tfr permutation tests
"""

from collections import OrderedDict

import numpy as np
import matplotlib.pyplot as plt
import mne

from meggie.utilities.plotting import color_cycle
from meggie.utilities.channels import get_channels_by_type
from meggie.utilities.units import get_power_unit
from meggie_statistics.utilities.stats import prepare_data_for_permutation
from meggie_statistics.utilities.stats import permutation_analysis
from meggie_statistics.utilities.stats import report_permutation_results
from meggie_statistics.utilities.stats import plot_permutation_results


def run_permutation_test(
    experiment,
    window,
    selected_name,
    groups,
    time_limits,
    frequency_limits,
    location_limits,
    threshold,
    significance,
    n_permutations,
    design,
):
    """Runs permutation test computation and reports the results."""
    if (
        location_limits[0] != "ch_type"
        and frequency_limits is not None
        and time_limits is not None
    ):
        raise Exception(
            "These are cluster permutation tests. Cannot run with all location, frequency and time limits set."
        )

    tfr_item = experiment.active_subject.tfr[selected_name]
    conditions = list(tfr_item.content.keys())
    groups = OrderedDict(sorted(groups.items()))
    times = tfr_item.times
    freqs = tfr_item.freqs

    chs_by_type = get_channels_by_type(tfr_item.info)
    if location_limits[0] == "ch_type":
        ch_type = location_limits[1]
    elif location_limits[0] == "ch_group":
        ch_type = location_limits[1][0]
    else:
        ch_type = [
            key for key, vals in chs_by_type.items() if location_limits[1] in vals
        ][0]

    info, data, adjacency = prepare_data_for_permutation(
        experiment,
        design,
        groups,
        "tfr",
        selected_name,
        location_limits,
        time_limits,
        frequency_limits,
        data_format=("locations", "freqs", "times"),
        do_meanwhile=window.update_ui,
    )

    results = permutation_analysis(
        data,
        design,
        conditions,
        groups,
        threshold,
        adjacency,
        n_permutations,
        do_meanwhile=window.update_ui,
    )

    report_permutation_results(
        results,
        design,
        selected_name,
        significance,
        location_limits=location_limits,
        time_limits=time_limits,
        frequency_limits=frequency_limits,
    )

    if design == "within-subjects":
        title_template = "Cluster {0} for group {1} (p {2})"
    else:
        title_template = "Cluster {0} for condition {1} (p {2})"

    def time_fun(cluster_idx, cluster, pvalue, res_key):
        fig, ax = plt.subplots()
        if design == "within-subjects":
            colors = color_cycle(len(conditions))
            for cond_idx, condition in enumerate(conditions):
                Y = np.mean(data[res_key][cond_idx], axis=0)
                Y = np.mean(Y[np.unique(cluster[0])], axis=0)
                Y = np.mean(Y[:, np.unique(cluster[-1])], axis=1)
                ax.plot(times, Y, label=condition, color=colors[cond_idx])
        else:
            colors = color_cycle(len(groups))
            for group_idx, (group_key, group) in enumerate(groups.items()):
                Y = np.mean(data[res_key][group_idx], axis=0)
                Y = np.mean(Y[np.unique(cluster[0])], axis=0)
                Y = np.mean(Y[:, np.unique(cluster[-1])], axis=1)
                ax.plot(times, Y, label=condition, color=colors[group_idx])

        fig.suptitle(title_template.format(cluster_idx + 1, res_key, pvalue))
        fig.canvas.manager.set_window_title("Cluster time course")

        ax.legend()
        ax.set_xlabel("Time (s)")
        ax.set_ylabel("Power ({})".format(get_power_unit(ch_type, log=False)))

        ax.axhline(0, color="black")
        ax.axvline(0, color="black")

        tmin = np.min(times[cluster[1]])
        tmax = np.max(times[cluster[1]])
        ax.axvspan(tmin, tmax, alpha=0.5, color="blue")

    def frequency_fun(cluster_idx, cluster, pvalue, res_key):
        fig, ax = plt.subplots()
        if design == "within-subjects":
            colors = color_cycle(len(conditions))
            for cond_idx, condition in enumerate(conditions):
                Y = np.mean(data[res_key][cond_idx], axis=0)
                Y = np.mean(Y[:, np.unique(cluster[1]), :], axis=1)
                Y = np.mean(Y[:, np.unique(cluster[-1])], axis=1)
                ax.plot(freqs, Y, label=condition, color=colors[cond_idx])
        else:
            colors = color_cycle(len(groups))
            for group_idx, (group_key, group) in enumerate(groups.items()):
                Y = np.mean(data[res_key][group_idx], axis=0)
                Y = np.mean(Y[:, np.unique(cluster[1]), :], axis=1)
                Y = np.mean(Y[:, np.unique(cluster[-1])], axis=1)
                ax.plot(freqs, Y, label=condition, color=colors[group_idx])

        fig.suptitle(title_template.format(cluster_idx + 1, res_key, pvalue))
        fig.canvas.manager.set_window_title("Cluster spectrum")

        ax.legend()
        ax.set_xlabel("Frequency (Hz)")
        ax.set_ylabel("Power ({})".format(get_power_unit(ch_type, log=False)))
        fmin = np.min(freqs[cluster[0]])
        fmax = np.max(freqs[cluster[0]])
        ax.axvspan(fmin, fmax, alpha=0.5, color="blue")

    def location_fun(cluster_idx, cluster, pvalue, res_key):
        map_ = [1 if idx in cluster[-1] else 0 for idx in range(len(info["ch_names"]))]

        fig, ax = plt.subplots()
        ch_type = location_limits[1]
        mne.viz.plot_topomap(
            np.array(map_),
            info,
            vlim=(0, 1),
            cmap="Reds",
            axes=ax,
            ch_type=ch_type,
            contours=0,
        )

        fig.suptitle(title_template.format(cluster_idx + 1, res_key, pvalue))
        fig.canvas.manager.set_window_title("Cluster topomap")

    plot_permutation_results(
        results,
        significance,
        window,
        location_limits=location_limits,
        frequency_limits=frequency_limits,
        time_limits=time_limits,
        location_fun=location_fun,
        frequency_fun=frequency_fun,
        time_fun=time_fun,
    )



from setuptools import setup

setup(
    name='meggie_statistics',
    version='0.3.1',
    description='Statistics plugin for Meggie',
    license='BSD',
    packages=['meggie_statistics'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'meggie>=1.5.0'
    ]
)



import tempfile

import mne

from meggie.datatypes.evoked.evoked import Evoked

from meggie_statistics.utilities.stats import prepare_data_for_permutation
from meggie_statistics.utilities.stats import permutation_analysis
from meggie.utilities.testing import create_evoked_conditions_experiment


def test_permutation_analysis():
    with tempfile.TemporaryDirectory() as dirpath:
        # generate dataset with generate_datasets to tmp folder (n=10)
        experiment = create_evoked_conditions_experiment(
            dirpath, "permutation_test_experiment", n_subjects=10
        )

        for subject_name, subject in experiment.subjects.items():
            raw = subject.get_raw()

            raw.resample(100)
            subject.save()

            events = mne.find_events(raw)
            evoked_la = mne.Epochs(raw, events, event_id=1).average()
            evoked_ra = mne.Epochs(raw, events, event_id=2).average()
            content = {"la": evoked_la, "ra": evoked_ra}
            meggie_evoked = Evoked(
                "Evoked", subject.evoked_directory, {}, content=content
            )
            subject.add(meggie_evoked, "evoked")

        experiment.activate_subject(subject_name)

        design = "within-subjects"
        conditions = ["la", "ra"]
        groups = {"1": list(experiment.subjects.keys())}
        location_limits = ("ch_type", "grad")
        time_limits = None
        threshold = 0.01
        n_permutations = 1000
        info, data, adjacency = prepare_data_for_permutation(
            experiment=experiment,
            design=design,
            groups=groups,
            item_type="evoked",
            item_name="Evoked",
            location_limits=location_limits,
            time_limits=time_limits,
            frequency_limits=None,
            data_format=("locations", "times"),
            no_threading=True,
        )

        results = permutation_analysis(
            data=data,
            design=design,
            conditions=conditions,
            groups=groups,
            threshold=threshold,
            adjacency=adjacency,
            n_permutations=n_permutations,
            no_threading=True,
            random_state=10,
        )

        # Correct cluster shape
        assert results["1"][0].shape == (71, 202)
